package com.pnlyy.midi_support;

import android.content.Context;
import android.content.pm.PackageManager;
import android.media.midi.MidiManager;
import android.os.Build;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.OnLifecycleEvent;
import com.pnlyy.midi_support.interfaces.OnMidiData;

/**
 * Midi 键盘(输出)设备监听，单设备
 */
public class MidiDeviceHelper implements LifecycleObserver {

    private final MidiSupportImpl<?> IMPL;

    /**
     * @param lifecycle 无需手动调用destroy
     * @param receiver 数据接收器
     * @param onOTGEvent 事件监听
     */
    public MidiDeviceHelper(
            @NonNull Context context,
            @Nullable Lifecycle lifecycle,
            @NonNull OnMidiData receiver,
            @NonNull OnOTGEvent onOTGEvent) {

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M &&
                context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_MIDI)) {
            MidiManager mMidiManager = (MidiManager) context.getSystemService(Context.MIDI_SERVICE);
            IMPL = new MidiSupportAboveM(mMidiManager, receiver, onOTGEvent);
            //若支持MidiManager 禁用 DeviceMonitorActivity
            //DeviceMonitorActivity.setEnabled(context, false);
        } else {
            IMPL = new MidiSupportUnderM(context, receiver, onOTGEvent);
        }
        if (lifecycle!=null) {
            lifecycle.addObserver(this);
        }
    }

    /**
     * 初始化
     * 设备监听
     */
    public void init() {
        IMPL.init();
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    public void onDestroy() {
        IMPL.onDestroy();
    }

    /**
     * 是否与设备建立连接
     */
    public boolean isConnected() {
        return IMPL.isConnected();
    }

    /**
     * 重新连接
     * 设备可能已连接 但未建立通信
     */
    public void reconnect() {
        IMPL.reconnect();
    }

    public interface OnOTGEvent {
        void onOTGDeviceDisconnect();

        void onOTGDeviceConnect(@NonNull Object device);

        void onOTGDeviceError(Throwable e);
    }

    public static boolean isSupportOTG(Context context) {
        return context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST);
    }
}
