package com.aysd.midi_otg_plugin

import android.content.ContentValues.TAG
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.annotation.NonNull
import com.pnlyy.midi_support.MidiDeviceHelper
import com.pnlyy.midi_support.interfaces.OnMidiData
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


private val mainHandler: Handler = Handler()

/** MidiOtgPlugin */
class MidiOtgPlugin: FlutterPlugin, MethodCallHandler, MidiDeviceHelper.OnOTGEvent, OnMidiData {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var midiHelper : MidiDeviceHelper

  private var mainHandler = Handler(Looper.getMainLooper());

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "midi_otg_plugin")
    midiHelper = MidiDeviceHelper(flutterPluginBinding.applicationContext, null, this, this)
    midiHelper.init()
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getPlatformVersion") {
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
      appendLog("设备断开连接" + "Android ${android.os.Build.VERSION.RELEASE}")
    } else if (call.method.equals("getBatteryLevel")) {
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
      appendLog("设备断开连接" + "Android ${android.os.Build.VERSION.RELEASE}")
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onOTGDeviceDisconnect() {
    appendLog("设备断开连接")
  }

  override fun onOTGDeviceConnect(device: Any) {
    appendLog("设备连接：$device")
  }

  override fun onOTGDeviceError(e: Throwable?) {
    appendLog("设备连接失败：$e")
  }

  override fun onData(
          data: ByteArray,
          offset: Int,
          count: Int,
          timestamp: Long
  ) {
  mainHandler.post {
    //在Android端调用Flutter中的getInputParams方法，参数是以Json的格式传递。这里是无参方法使用null
    channel.invokeMethod("get_message", null, object : Result {
      override fun success(o: Any?) {
        Log.d(TAG, "get_message:${
          data.joinToString(
                  prefix = "[", postfix = "]"
          ) { (it.toInt() and 0xff).toString(16) }
        }")
      }

      override fun error(s: String?, s1: String?, o: Any?) {
        TODO("not implemented")
      }

      override fun notImplemented() {
        TODO("not implemented")
      }
    })
  }




    appendLog("midi data: " + data.joinToString(
            prefix = "[", postfix = "]"
    ) { (it.toInt() and 0xff).toString(16) })

  }

  private fun appendLog(s: String) {
    Log.d("flutter: ", s)
  }

}
