#import <Foundation/Foundation.h>

typedef enum
{
    LOG_DEBUG = 1,
    LOG_INFO,
    LOG_WARN,
    LOG_ERROR,
    LOG_FATAL,
    LOG_NONE
} LoggingLevel;

#define objc_log_C(level, format, ...){\
[Obj_Logging appendLog:(int)level fromFunction:__FUNCTION__ fromFile:__FILE__ fromLine:__LINE__ withFormat:(const char*)format, ##__VA_ARGS__];\
}
#define objc_log_NSString(level, format){\
[Obj_Logging appendNSStringLog:(int)level fromFunction:__FUNCTION__ fromFile:__FILE__ fromLine:__LINE__ withNSString:format];\
}

@interface Obj_Logging : NSObject
{

}

+ (NSString*)getCurrentTime;

+ (void)initWithLevel:(int)level;

+ (void) appendLog:(int)level
      fromFunction:(const char*)strFunc
          fromFile:(const char*)strFile
          fromLine:(int) line
        withFormat:(const char*)format, ...;


+ (void) appendNSStringLog:(int)level
      fromFunction:(const char*)strFunc
          fromFile:(const char*)strFile
          fromLine:(int) line
        withNSString:(NSString*)strNSString;
@end

//How to use
/*
 In app entrance:
 [Obj_Logging initWithLevel:LOG_DEBUG];
 
 To log data:
 objc_log_C(LOG_INFO, "Start Log:");
 objc_log_NSString(LOG_INFO, @"Start Log:");
*/