#import "Obj_Logging.h"

static int s_logLevel = LOG_DEBUG;
static FILE *s_fp = nil;
static dispatch_queue_t s_queueLog = nil;
static int s_countThread = 0;

@implementation Obj_Logging


+ (NSString*)getCurrentTime {
    NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    return [formatter stringFromDate:[NSDate date]];
}

+ (void)initWithLevel:(int)level {
    NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *strDocumentDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];

    NSString *filePath = [strDocumentDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.txt", [Obj_Logging getCurrentTime]]];
    if(![fileManager fileExistsAtPath:filePath])
        [fileManager createFileAtPath:filePath contents:nil attributes:nil];
    
    s_queueLog = dispatch_queue_create("logQueue", NULL);
    s_fp = fopen([filePath UTF8String], "wb");
    s_logLevel = level;
    
    objc_log_C(LOG_INFO, "Start Log:");
}

+ (void) appendLog:(int)level
      fromFunction:(const char*)strFunc
          fromFile:(const char*)strFile
          fromLine:(int) line
        withFormat:(const char*)format, ...
{
    if (level < s_logLevel)
        return;
    
    if ([[[NSThread currentThread] name] isEqualToString:@""])
    {
        s_countThread++;
        [[NSThread currentThread] isMainThread] ?
            [[NSThread currentThread] setName:[NSString stringWithFormat:@"Main Thread %d", s_countThread]] :
            [[NSThread currentThread] setName:[NSString stringWithFormat:@"Non-Main Thread %d", s_countThread]];
    }

    dispatch_sync(s_queueLog, ^
    {
        const char *cstr = [[NSString stringWithFormat:@"%@ (%@) %s in %@(%d)----%s\n", [Obj_Logging getCurrentTime], [[NSThread currentThread] name], strFunc, [[NSString stringWithUTF8String:strFile] lastPathComponent], line, format] UTF8String];
        fwrite(cstr, strlen(cstr), 1, s_fp);
        fflush(s_fp);
    });
}

+ (void) appendNSStringLog:(int)level
              fromFunction:(const char*)strFunc
                  fromFile:(const char*)strFile
                  fromLine:(int) line
              withNSString:(NSString*)strNSString
{
    [Obj_Logging appendLog:level
              fromFunction:strFunc
                  fromFile:strFile
                  fromLine:line
                withFormat:[strNSString UTF8String]];
}
@end

